@@ -1,9 +1,9 @@
 /*
- * "$Id: dist.c,v 1.38 2001/07/05 16:57:22 mike Exp $"
+ * "$Id: dist.c,v 1.49 2002/10/18 14:57:48 mike Exp $"
  *
  *   Distribution functions for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2001 by Easy Software Products.
+ *   Copyright 1999-2002 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,14 +21,21 @@
  *   add_depend()      - Add a dependency to the distribution...
  *   add_file()        - Add a file to the distribution.
  *   free_dist()       - Free memory used by a distribution.
+ *   getoption()       - Get an option from a file.
  *   get_platform()    - Get the operating system information...
+ *   get_runlevels()   - Get the run levels for the specified init script.
+ *   get_start()       - Get the start number for an init script.
+ *   get_stop()        - Get the stop number for an init script.
  *   read_dist()       - Read a software distribution.
  *   sort_dist_files() - Sort the files in the distribution.
  *   add_string()      - Add a command to an array of commands...
  *   compare_files()   - Compare the destination filenames.
  *   free_strings()    - Free memory used by the array of strings.
  *   expand_name()     - Expand a filename with environment variables.
+ *   get_file()        - Read a file into a string...
+ *   get_inline()      - Read inline lines into a string...
  *   get_line()        - Get a line from a file, filtering for uname lines...
+ *   get_string()      - Get a delimited string from a line.
  *   get_vernumber()   - Convert a version string to a number...
  *   patmatch()        - Pattern matching...
  */
@@ -54,29 +61,63 @@ extern int	gethostname(char *, int);
  * Local functions...
  */
 
-static int	add_string(int num_strings, char ***strings, char *string);
+static int	add_string(int num_strings, char ***strings, FILE *fp,
+		           char *string);
 static int	compare_files(const file_t *f0, const file_t *f1);
 static void	expand_name(char *buffer, char *name);
 static void	free_strings(int num_strings, char **strings);
+static char	*get_file(const char *filename, char *buffer, int size);
+static char	*get_inline(const char *term, FILE *fp, char *buffer, int size);
 static char	*get_line(char *buffer, int size, FILE *fp,
 		          struct utsname *platform, const char *format,
 			  int *skip);
+static char	*get_string(char **src, char *dst, int dstsize);
 static int	get_vernumber(const char *version);
 static int	patmatch(const char *, const char *);
 
 
+/*
+ * Conditional "skip" bits...
+ */
+
+#define SKIP_SYSTEM	1	/* Not the right system */
+#define SKIP_FORMAT	2	/* Not the right format */
+#define SKIP_IF		4	/* Set if the current #if was not satisfied */
+#define SKIP_IFACTIVE	8	/* Set if we're in an #if */
+#define SKIP_IFSAT	16	/* Set if an #if statement has been satisfied */
+#define SKIP_MASK	7	/* Bits to look at */
+
+
 /*
  * 'add_command()' - Add a command to the distribution...
  */
 
 void
 add_command(dist_t     *dist,		/* I - Distribution */
+            FILE       *fp,		/* I - Distribution file */
             char       type,		/* I - Command type */
 	    const char *command)	/* I - Command string */
 {
   command_t	*temp;			/* New command */
+  char		buf[16384];		/* File import buffer */
 
 
+  if (strncmp(command, "<<", 2) == 0)
+  {
+    for (command += 2; isspace(*command); command ++);
+
+    command = get_inline(command, fp, buf, sizeof(buf));
+  }
+  else if (command[0] == '<' && command[1])
+  {
+    for (command ++; isspace(*command); command ++);
+
+    command = get_file(command, buf, sizeof(buf));
+  }
+
+  if (command == NULL)
+    return;
+
   if (dist->num_commands == 0)
     temp = malloc(sizeof(command_t));
   else
@@ -90,9 +131,8 @@ add_command(dist_t     *dist,		/* I - Distribution */
 
   dist->commands = temp;
   temp           += dist->num_commands;
-
-  temp->type    = type;
-  temp->command = strdup(command);
+  temp->type     = type;
+  temp->command  = strdup(command);
 
   if (temp->command == NULL)
   {
@@ -288,6 +328,48 @@ free_dist(dist_t *dist)		/* I - Distribution to free */
 }
 
 
+
+/*
+ * 'getoption()' - Get an option from a file.
+ */
+
+const char *				/* O - Value */
+get_option(file_t     *file,		/* I - File */
+           const char *name,		/* I - Name of option */
+	   const char *defval)		/* I - Default value of option */
+{
+  char		*ptr;			/* Pointer to option */
+  static char	option[256];		/* Copy of file option */
+
+
+ /*
+  * See if the option exists...
+  */
+
+  snprintf(option, sizeof(option), "%s(", name);
+
+  if ((ptr = strstr(file->options, option)) == NULL)
+    return (defval);
+
+ /*
+  * Yes, copy the value and truncate at the first ")"...
+  */
+
+  ptr += strlen(option);
+  strcpy(option, ptr);	/* option and file->options are of equal size */
+
+  if ((ptr = strchr(option, ')')) != NULL)
+  {
+    *ptr = '\0';
+    return (option);
+  }
+  else
+    return (defval);
+}
+
+
+
+
 /*
  * 'get_platform()' - Get the operating system information...
  */
@@ -312,7 +394,7 @@ get_platform(struct utsname *platform)	/* O - Platform info */
   strcpy(platform->machine, "mips");
 #elif defined(__hpux)
   strcpy(platform->machine, "hppa");
-#elif defined(_AIX)
+#elif defined(_AIX) || defined(__APPLE__)
   strcpy(platform->machine, "powerpc");
 #else
   for (temp = platform->machine; *temp != '\0'; temp ++)
@@ -396,6 +478,62 @@ get_platform(struct utsname *platform)	/* O - Platform info */
 }
 
 
+/*
+ * 'get_runlevels()' - Get the run levels for the specified init script.
+ */
+
+const char *				/* O - Run levels */
+get_runlevels(file_t     *file,		/* I - File */
+              const char *deflevels)	/* I - Default run levels */
+{
+  const char	*runlevels;		/* Pointer to runlevels option */
+
+
+  if ((runlevels = strstr(file->options, "runlevel(")) != NULL)
+    runlevels += 9;
+  else
+    runlevels = deflevels;
+
+  return (runlevels);
+}
+
+
+/*
+ * 'get_start()' - Get the start number for an init script.
+ */
+
+int					/* O - Start number */
+get_start(file_t *file,			/* I - File */
+          int    defstart)		/* I - Default start number */
+{
+  const char	*start;			/* Pointer to start option */
+
+
+  if ((start = strstr(file->options, "start(")) != NULL)
+    return (atoi(start + 6));
+  else
+    return (defstart);
+}
+
+
+/*
+ * 'get_stop()' - Get the stop number for an init script.
+ */
+
+int					/* O - Start number */
+get_stop(file_t *file,			/* I - File */
+          int    defstop)		/* I - Default stop number */
+{
+  const char	*stop;			/* Pointer to stop option */
+
+
+  if ((stop = strstr(file->options, "stop(")) != NULL)
+    return (atoi(stop + 5));
+  else
+    return (defstop);
+}
+
+
 /*
  * 'read_dist()' - Read a software distribution.
  */
@@ -415,7 +553,8 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 		pattern[256],	/* Pattern for source files */
 		user[32],	/* User */
 		group[32],	/* Group */
-		*temp;		/* Temporary pointer */
+		*temp,		/* Temporary pointer */
+		options[256];	/* File options */
   int		mode,		/* File permissions */
 		skip;		/* 1 = skip files, 0 = archive files */
   dist_t	*dist;		/* Distribution data */
@@ -495,22 +634,23 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 	}
 	else if (strcmp(line, "%description") == 0)
 	  dist->num_descriptions =
-	      add_string(dist->num_descriptions, &(dist->descriptions), temp);
+	      add_string(dist->num_descriptions, &(dist->descriptions), 
+	                 listfiles[listlevel], temp);
 	else if (strcmp(line, "%preinstall") == 0)
-          add_command(dist, COMMAND_PRE_INSTALL, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_PRE_INSTALL, temp);
 	else if (strcmp(line, "%install") == 0 ||
 	         strcmp(line, "%postinstall") == 0)
-          add_command(dist, COMMAND_POST_INSTALL, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_POST_INSTALL, temp);
 	else if (strcmp(line, "%remove") == 0 ||
 	         strcmp(line, "%preremove") == 0)
-          add_command(dist, COMMAND_PRE_REMOVE, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_PRE_REMOVE, temp);
 	else if (strcmp(line, "%postremove") == 0)
-          add_command(dist, COMMAND_POST_REMOVE, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_POST_REMOVE, temp);
 	else if (strcmp(line, "%prepatch") == 0)
-          add_command(dist, COMMAND_PRE_PATCH, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_PRE_PATCH, temp);
 	else if (strcmp(line, "%patch") == 0 ||
 	         strcmp(line, "%postpatch") == 0)
-          add_command(dist, COMMAND_POST_PATCH, temp);
+          add_command(dist, listfiles[listlevel], COMMAND_POST_PATCH, temp);
         else if (strcmp(line, "%product") == 0)
 	{
           if (!dist->product[0])
@@ -634,13 +774,55 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 	  }
 	}
       }
-      else if (sscanf(line, "%c%o%15s%15s%254s%254s", &type, &mode, user, group,
-        	      dst, src) < 5)
-	fprintf(stderr, "epm: Bad line - %s\n", line);
       else
       {
+        type = line[0];
+	if (!isspace(line[1]))
+	{
+	  fprintf(stderr, "epm: Expected whitespace after file type - %s\n",
+	          line);
+	  continue;
+	}
+
+        temp = line + 2;
+	mode = strtol(temp, &temp, 8);
+	if (temp == (line + 2))
+	{
+	  fprintf(stderr, "epm: Expected file permissions after file type - %s\n",
+	          line);
+	  continue;
+	}
+
+        if (get_string(&temp, user, sizeof(user)) == NULL)
+	{
+	  fprintf(stderr, "epm: Expected user after file permissions - %s\n",
+	          line);
+	  continue;
+	}
+
+        if (get_string(&temp, group, sizeof(group)) == NULL)
+	{
+	  fprintf(stderr, "epm: Expected group after user - %s\n",
+	          line);
+	  continue;
+	}
+
+        if (get_string(&temp, dst, sizeof(dst)) == NULL)
+	{
+	  fprintf(stderr, "epm: Expected destination after group - %s\n",
+	          line);
+	  continue;
+	}
+
+        get_string(&temp, src, sizeof(src));
+
+        get_string(&temp, options, sizeof(options));
+
 	if (tolower(type) == 'd' || type == 'R')
+	{
+	  strcpy(options, src);
 	  src[0] = '\0';
+	}
 
 #ifdef __osf__ /* Remap group "sys" to "system" */
         if (strcmp(group, "sys") == 0)
@@ -714,6 +896,7 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 	      strncat(file->dst, dent->d_name, sizeof(file->dst) - 1);
 	      strcpy(file->user, user);
 	      strcpy(file->group, group);
+	      strcpy(file->options, options);
 	    }
 
             closedir(dir);
@@ -733,6 +916,7 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 	  strcpy(file->dst, dst);
 	  strcpy(file->user, user);
 	  strcpy(file->group, group);
+	  strcpy(file->options, options);
 	}
       }
     }
@@ -804,14 +988,49 @@ sort_dist_files(dist_t *dist)	/* I - Distribution to sort */
 static int			/* O - Number of strings */
 add_string(int  num_strings,	/* I - Number of strings */
            char ***strings,	/* O - Pointer to string pointer array */
+           FILE *fp,		/* I - File to read from for inline text */
            char *string)	/* I - String to add */
 {
+  char		**temp,		/* Temporary pointer to string array */
+		buf[16384];	/* File import buffer */
+
+
+  if (strncmp(string, "<<", 2) == 0)
+  {
+    for (string += 2; isspace(*string); string ++);
+
+    string = get_inline(string, fp, buf, sizeof(buf));
+  }
+  else if (string[0] == '<' && string[1])
+  {
+    for (string ++; isspace(*string); string ++);
+
+    string = get_file(string, buf, sizeof(buf));
+  }
+
+  if (string == NULL)
+    return (num_strings);
+
   if (num_strings == 0)
-    *strings = (char **)malloc(sizeof(char *));
+    temp = (char **)malloc(sizeof(char *));
   else
-    *strings = (char **)realloc(*strings, (num_strings + 1) * sizeof(char *));
+    temp = (char **)realloc(*strings, (num_strings + 1) * sizeof(char *));
+
+  if (temp == NULL)
+  {
+    perror("epm: Out of memory adding string");
+    return (num_strings);
+  }
+
+  *strings = temp;
+  temp     += num_strings;
+
+  if ((*temp = strdup(string)) == NULL)
+  {
+    perror("epm: Out of memory duplicating string");
+    return (num_strings);
+  }
 
-  (*strings)[num_strings] = strdup(string);
   return (num_strings + 1);
 }
 
@@ -911,6 +1130,112 @@ free_strings(int  num_strings,	/* I - Number of strings */
 }
 
 
+/*
+ * 'get_file()' - Read a file into a string...
+ */
+
+static char *			/* O  - Pointer to string or NULL on EOF */
+get_file(const char *filename,	/* I  - File to read from */
+         char       *buffer,	/* IO - String buffer */
+	 int        size)	/* I  - Size of string buffer */
+{
+  FILE		*fp;		/* File buffer */
+  struct stat	info;		/* File information */
+
+
+  if (stat(filename, &info))
+  {
+    fprintf(stderr, "epm: Unable to stat \"%s\" - %s\n", filename,
+            strerror(errno));
+    return (NULL);
+  }
+
+  if (info.st_size > (size - 1))
+  {
+    fprintf(stderr, "epm: File \"%s\" is too large (%d bytes) for buffer (%d bytes)\n",
+            filename, (int)info.st_size, size - 1);
+    return (NULL);
+  }
+
+  if ((fp = fopen(filename, "r")) == NULL)
+  {
+    fprintf(stderr, "epm: Unable to open \"%s\" - %s\n", filename,
+            strerror(errno));
+    return (NULL);
+  }
+
+  if ((fread(buffer, 1, info.st_size, fp)) < info.st_size)
+  {
+    fprintf(stderr, "epm: Unable to read \"%s\" - %s\n", filename,
+            strerror(errno));
+    fclose(fp);
+    return (NULL);
+  }
+
+  fclose(fp);
+
+  if (buffer[info.st_size - 1] == '\n')
+    buffer[info.st_size - 1] = '\0';
+  else
+    buffer[info.st_size] = '\0';
+
+  return (buffer);
+}
+
+
+/*
+ * 'get_inline()' - Read inline lines into a string...
+ */
+
+static char *			/* O  - Pointer to string or NULL on EOF */
+get_inline(const char *term,	/* I  - Termination string */
+           FILE       *fp,	/* I  - File to read from */
+           char       *buffer,	/* IO - String buffer */
+	   int        size)	/* I  - Size of string buffer */
+{
+  char	*bufptr;		/* Pointer into buffer */
+  int	termlen;		/* Length of termination string */
+  int	linelen;		/* Length of line */
+
+
+  bufptr  = buffer;
+  termlen = strlen(term);
+
+  if (termlen == 0)
+    return (NULL);
+
+  while (fgets(bufptr, size, fp) != NULL)
+  {
+    if (strncmp(bufptr, term, termlen) == 0 && bufptr[termlen] == '\n')
+    {
+      *bufptr = '\0';
+      break;
+    }
+
+    linelen = strlen(bufptr);
+    size    -= linelen;
+    bufptr  += linelen;
+
+    if (size < 2)
+    {
+      fputs("epm: Inline script too long!\n", stderr);
+      break;
+    }
+  }
+
+  if (bufptr > buffer)
+  {
+    bufptr --;
+    if (*bufptr == '\n')
+      *bufptr = '\0';
+
+    return (buffer);
+  }
+  else
+    return (NULL);
+}
+
+
 /*
  * 'get_line()' - Get a line from a file, filtering for uname lines...
  */
@@ -923,14 +1248,15 @@ get_line(char           *buffer,	/* I - Buffer to read into */
          const char     *format,	/* I - Distribution format */
 	 int            *skip)		/* IO - Skip lines? */
 {
-  int	op,				/* Operation (0 = OR, 1 = AND) */
-	namelen,			/* Length of system name + version */
-	len,				/* Length of string */
-	match;				/* 1 = match, 0 = not */
-  char	*ptr,				/* Pointer into value */
-	*bufptr,			/* Pointer into buffer */
-	namever[255],			/* Name + version */
-	value[255];			/* Value string */
+  int		op,			/* Operation (0 = OR, 1 = AND) */
+		namelen,		/* Length of system name + version */
+		len,			/* Length of string */
+		match;			/* 1 = match, 0 = not */
+  char		*ptr,			/* Pointer into value */
+		*bufptr,		/* Pointer into buffer */
+		namever[255],		/* Name + version */
+		value[255];		/* Value string */
+  const char	*var;			/* Variable value */
 
 
   while (fgets(buffer, size, fp) != NULL)
@@ -943,7 +1269,7 @@ get_line(char           *buffer,	/* I - Buffer to read into */
       continue;
 
    /*
-    * See if this is a %system or %format line...
+    * See if this is a %system, %format, or conditional line...
     */
 
     if (strncmp(buffer, "%system ", 8) == 0)
@@ -952,7 +1278,7 @@ get_line(char           *buffer,	/* I - Buffer to read into */
       * Yes, do filtering based on the OS (+version)...
       */
 
-      *skip &= 2;
+      *skip &= ~SKIP_SYSTEM;
 
       if (strcmp(buffer + 8, "all\n") != 0)
       {
@@ -961,10 +1287,24 @@ get_line(char           *buffer,	/* I - Buffer to read into */
 	snprintf(namever, sizeof(namever), "%s-%s", platform->sysname,
 	         platform->release);
 
-        *skip |= *bufptr != '!';
+	while (isspace((int)*bufptr))
+	  bufptr ++;
+
+        if (*bufptr != '!')
+	  *skip |= SKIP_SYSTEM;
 
         while (*bufptr)
 	{
+	 /*
+          * Skip leading whitespace...
+	  */
+
+	  while (isspace((int)*bufptr))
+	    bufptr ++;
+
+          if (!*bufptr)
+	    break;
+
           if (*bufptr == '!')
 	  {
 	    op = 1;
@@ -973,13 +1313,12 @@ get_line(char           *buffer,	/* I - Buffer to read into */
 	  else
 	    op = 0;
 
-	  for (ptr = value; *bufptr && !isspace((int)*bufptr) &&
-	                        (ptr - value) < (sizeof(value) - 1);)
-	    *ptr++ = *bufptr++;
+	  for (ptr = value;
+	       *bufptr && !isspace((int)*bufptr) &&
+	           ptr < (value + sizeof(value) - 1);
+	       *ptr++ = *bufptr++);
 
 	  *ptr = '\0';
-	  while (isspace((int)*bufptr))
-	    bufptr ++;
 
           if (strncmp(value, "dunix", 5) == 0)
 	    memcpy(value, "tru64", 5); /* Keep existing nul/version */
@@ -992,7 +1331,7 @@ get_line(char           *buffer,	/* I - Buffer to read into */
           if (len < namelen)
 	    match = 0;
 	  else
-	    match = strncasecmp(value, namever, strlen(value)) == 0;
+	    match = strncasecmp(value, namever, strlen(value)) == 0 ? SKIP_SYSTEM : 0;
 
           if (op)
 	    *skip |= match;
@@ -1007,15 +1346,30 @@ get_line(char           *buffer,	/* I - Buffer to read into */
       * Yes, do filtering based on the distribution format...
       */
 
-      *skip &= 1;
+      *skip &= ~SKIP_FORMAT;
 
       if (strcmp(buffer + 8, "all\n") != 0)
       {
         bufptr = buffer + 8;
-        *skip  |= (*bufptr != '!') * 2;
+
+	while (isspace((int)*bufptr))
+	  bufptr ++;
+
+        if (*bufptr != '!')
+	  *skip  |= SKIP_FORMAT;
 
         while (*bufptr)
 	{
+	 /*
+          * Skip leading whitespace...
+	  */
+
+	  while (isspace((int)*bufptr))
+	    bufptr ++;
+
+          if (!*bufptr)
+	    break;
+
           if (*bufptr == '!')
 	  {
 	    op = 1;
@@ -1024,15 +1378,14 @@ get_line(char           *buffer,	/* I - Buffer to read into */
 	  else
 	    op = 0;
 
-	  for (ptr = value; *bufptr && !isspace((int)*bufptr) &&
-	                        (ptr - value) < (sizeof(value) - 1);)
-	    *ptr++ = *bufptr++;
+	  for (ptr = value;
+	       *bufptr && !isspace((int)*bufptr) &&
+	           ptr < (value + sizeof(value) - 1);
+	       *ptr++ = *bufptr++);
 
 	  *ptr = '\0';
-	  while (isspace((int)*bufptr))
-	    bufptr ++;
 
-	  match = (strcasecmp(value, format) == 0) * 2;
+	  match = (strcasecmp(value, format) == 0) ? SKIP_FORMAT : 0;
 
           if (op)
 	    *skip |= match;
@@ -1041,7 +1394,199 @@ get_line(char           *buffer,	/* I - Buffer to read into */
         }
       }
     }
-    else if (!*skip)
+    else if (strncmp(buffer, "%ifdef ", 7) == 0 ||
+             strncmp(buffer, "%elseifdef ", 11) == 0)
+    {
+     /*
+      * Yes, do filtering based on the presence of variables...
+      */
+
+      if ((*skip & SKIP_IFACTIVE) && buffer[1] != 'e')
+      {
+       /*
+        * Nested %if...
+	*/
+
+        fputs("epm: Warning, nested %ifdef's are not supported!\n", stderr);
+	continue;
+      }
+
+      if (buffer[1] == 'e')
+	bufptr = buffer + 11;
+      else
+	bufptr = buffer + 7;
+
+      *skip |= SKIP_IF | SKIP_IFACTIVE;
+
+      if (*skip & SKIP_IFSAT)
+        continue;
+
+      while (isspace((int)*bufptr))
+	bufptr ++;
+
+      if (*bufptr == '!')
+        *skip &= ~SKIP_IF;
+
+      while (*bufptr)
+      {
+       /*
+        * Skip leading whitespace...
+	*/
+
+	while (isspace((int)*bufptr))
+	  bufptr ++;
+
+        if (!*bufptr)
+	  break;
+
+       /*
+        * Get the variable name...
+	*/
+
+        if (*bufptr == '!')
+	{
+	  op = 1;
+	  bufptr ++;
+	}
+	else
+	  op = 0;
+
+	for (ptr = value;
+	     *bufptr && !isspace((int)*bufptr) &&
+	         ptr < (value + sizeof(value) - 1);
+	     *ptr++ = *bufptr++);
+
+	*ptr = '\0';
+
+	match = (getenv(value) != NULL) ? SKIP_IF : 0;
+
+        if (op)
+	  *skip |= match;
+	else
+	  *skip &= ~match;
+
+        if (match)
+	  *skip |= SKIP_IFSAT;
+      }
+    }
+    else if (strncmp(buffer, "%if ", 4) == 0 ||
+             strncmp(buffer, "%elseif ", 8) == 0)
+    {
+     /*
+      * Yes, do filtering based on the value of variables...
+      */
+
+      if ((*skip & SKIP_IFACTIVE) && buffer[1] != 'e')
+      {
+       /*
+        * Nested %if...
+	*/
+
+        fputs("epm: Warning, nested %if's are not supported!\n", stderr);
+	continue;
+      }
+
+      if (buffer[1] == 'e')
+        bufptr = buffer + 8;
+      else
+	bufptr = buffer + 4;
+
+      *skip |= SKIP_IF | SKIP_IFACTIVE;
+
+      if (*skip & SKIP_IFSAT)
+        continue;
+
+      while (isspace((int)*bufptr))
+	bufptr ++;
+
+      if (*bufptr == '!')
+        *skip &= ~SKIP_IF;
+
+      while (*bufptr)
+      {
+       /*
+        * Skip leading whitespace...
+	*/
+
+	while (isspace((int)*bufptr))
+	  bufptr ++;
+
+        if (!*bufptr)
+	  break;
+
+       /*
+        * Get the variable name...
+	*/
+
+        if (*bufptr == '!')
+	{
+	  op = 1;
+	  bufptr ++;
+	}
+	else
+	  op = 0;
+
+	for (ptr = value;
+	     *bufptr && !isspace((int)*bufptr) &&
+	         ptr < (value + sizeof(value) - 1);
+	     *ptr++ = *bufptr++);
+
+	*ptr = '\0';
+
+        match = ((var = getenv(value)) != NULL && *var) ? SKIP_IF : 0;
+
+        if (op)
+	  *skip |= match;
+	else
+	  *skip &= ~match;
+
+        if (match)
+	  *skip |= SKIP_IFSAT;
+      }
+    }
+    else if (strcmp(buffer, "%else\n") == 0)
+    {
+     /*
+      * Handle "else" condition of %ifdef statement...
+      */
+
+      if (!(*skip & SKIP_IFACTIVE))
+      {
+       /*
+        * No matching %if/%ifdef statement...
+	*/
+
+        fputs("epm: Warning, no matching %if or %ifdef for %else!\n", stderr);
+	break;
+      }
+
+      if (*skip & SKIP_IFSAT)
+        *skip |= SKIP_IF;
+      else
+      {
+	*skip &= ~SKIP_IF;
+	*skip |= SKIP_IFSAT;
+      }
+    }
+    else if (strcmp(buffer, "%endif\n") == 0)
+    {
+     /*
+      * Cancel any filtering based on environment variables.
+      */
+
+      if (!(*skip & SKIP_IFACTIVE))
+      {
+       /*
+        * No matching %if/%ifdef statement...
+	*/
+
+        fputs("epm: Warning, no matching %if or %ifdef for %endif!\n", stderr);
+	break;
+      }
+
+      *skip &= ~(SKIP_IF | SKIP_IFACTIVE | SKIP_IFSAT);
+    }
+    else if (!(*skip & SKIP_MASK))
     {
      /*
       * Otherwise strip any trailing newlines and return the string!
@@ -1058,6 +1603,135 @@ get_line(char           *buffer,	/* I - Buffer to read into */
 }
 
 
+/*
+ * 'get_string()' - Get a delimited string from a line.
+ */
+
+static char *				/* O  - String or NULL */
+get_string(char **src,			/* IO - Source string */
+           char *dst,			/* O  - Destination string */
+	   int  dstsize)		/* I  - Size of destination string */
+{
+  char	*srcptr,			/* Current source pointer */
+	*dstptr,			/* Current destination pointer */
+	*dstend,			/* End of destination string */
+	quote;				/* Quoting char */
+
+
+ /*
+  * Initialize things...
+  */
+
+  srcptr = *src;
+  dstptr = dst;
+  dstend = dst + dstsize - 1;
+
+  *dstptr = '\0';
+
+ /*
+  * Skip leading whitespace...
+  */
+
+  while (isspace(*srcptr))
+    srcptr ++;
+
+  if (!*srcptr)
+  {
+    *src = srcptr;
+
+    return (NULL);
+  }
+
+ /*
+  * Grab the next string...
+  */
+
+  while (*srcptr && !isspace(*srcptr))
+  {
+    if (*srcptr == '\\')
+    {
+      srcptr ++;
+
+      if (!*srcptr)
+      {
+        fputs("epm: Expected character after backslash!\n", stderr);
+
+        *src = srcptr;
+        *dst = '\0';
+
+	return (NULL);
+      }
+    }
+    else if (*srcptr == '\'' || *srcptr == '\"')
+    {
+     /*
+      * Read a quoted string...
+      */
+
+      quote = *srcptr++;
+
+      while (*srcptr != quote && *srcptr)
+      {
+        if (*srcptr == '\\')
+	{
+	  srcptr ++;
+
+	  if (!*srcptr)
+	  {
+            fputs("epm: Expected character after backslash!\n", stderr);
+
+            *src = srcptr;
+            *dst = '\0';
+
+	    return (NULL);
+	  }
+	}
+
+	if (dstptr < dstend)
+	  *dstptr++ = *srcptr;
+
+	srcptr ++;
+      }
+
+      if (!*srcptr)
+      {
+        fprintf(stderr, "epm: Expected end quote %c!\n", quote);
+
+        *src = srcptr;
+	*dst = '\0';
+
+	return (NULL);
+      }
+
+      srcptr ++;
+      continue;
+    }
+
+    if (dstptr < dstend)
+      *dstptr++ = *srcptr;
+
+    srcptr ++;
+  }
+
+  *dstptr = '\0';
+
+ /*
+  * Skip leading whitespace...
+  */
+
+  while (isspace(*srcptr))
+    srcptr ++;
+
+ /*
+  * Return the string and string pointer...
+  */
+
+  *src = srcptr;
+
+  return (dst);
+}
+
+
 /*
  * 'get_vernumber()' - Convert a version string to a number...
  */
@@ -1234,6 +1908,14 @@ patmatch(const char *s,		/* I - String to match against */
       return (0);
   }
 
+ /*
+  * If we are at the end of the string, see if the pattern remaining is
+  * "*"...
+  */
+
+  while (*pat == '*')
+    pat ++;
+    
  /*
   * Done parsing the pattern and string; return 1 if the last character matches
   * and 0 otherwise...
@@ -1244,5 +1926,5 @@ patmatch(const char *s,		/* I - String to match against */
 
 
 /*
- * End of "$Id: dist.c,v 1.38 2001/07/05 16:57:22 mike Exp $".
+ * End of "$Id: dist.c,v 1.49 2002/10/18 14:57:48 mike Exp $".
  */
