@@ -1,9 +1,9 @@
 /*
- * "$Id: pkg.c,v 1.18 2001/06/26 16:22:22 mike Exp $"
+ * "$Id: pkg.c,v 1.23 2002/10/17 18:13:14 mike Exp $"
  *
  *   AT&T package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2001 by Easy Software Products.
+ *   Copyright 1999-2002 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   tarf_t	*tarfile;		/* Distribution file */
   time_t	curtime;		/* Current time info */
   struct tm	*curdate;		/* Current date info */
+  const char	*runlevels;		/* Run levels */
 
 
   if (Verbosity)
@@ -221,7 +222,7 @@ make_pkg(const char     *prodname,	/* I - Product short name */
 
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
       if (tolower(file->type) == 'i')
-	fprintf(fp, "/etc/init.d/%s start\n", file->dst);
+	qprintf(fp, "/etc/init.d/%s start\n", file->dst);
 
     fclose(fp);
   }
@@ -266,7 +267,7 @@ make_pkg(const char     *prodname,	/* I - Product short name */
 
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
       if (tolower(file->type) == 'i')
-	fprintf(fp, "/etc/init.d/%s stop\n", file->dst);
+	qprintf(fp, "/etc/init.d/%s stop\n", file->dst);
 
     for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
       if (c->type == COMMAND_PRE_REMOVE)
@@ -325,21 +326,34 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   for (i = 0; i < dist->num_files; i ++)
     if (tolower(dist->files[i].type) == 'i')
     {
-      file = add_file(dist);
-      file->type = 'l';
-      file->mode = 0;
-      strcpy(file->user, "root");
-      strcpy(file->group, "sys");
-      snprintf(file->src, sizeof(file->src), "../init.d/%s", dist->files[i].dst);
-      snprintf(file->dst, sizeof(file->dst), "/etc/rc0.d/K00%s", dist->files[i].dst);
+     /*
+      * Make symlinks for all of the selected run levels...
+      */
 
-      file = add_file(dist);
-      file->type = 'l';
-      file->mode = 0;
-      strcpy(file->user, "root");
-      strcpy(file->group, "sys");
-      snprintf(file->src, sizeof(file->src), "../init.d/%s", dist->files[i].dst);
-      snprintf(file->dst, sizeof(file->dst), "/etc/rc3.d/S99%s", dist->files[i].dst);
+      for (runlevels = get_runlevels(dist->files + i, "03");
+           isdigit(*runlevels);
+	   runlevels ++)
+      {
+	file = add_file(dist);
+	file->type = 'l';
+	file->mode = 0;
+	strcpy(file->user, "root");
+	strcpy(file->group, "sys");
+	snprintf(file->src, sizeof(file->src), "../init.d/%s",
+        	 dist->files[i].dst);
+
+        if (*runlevels == '0')
+	  snprintf(file->dst, sizeof(file->dst), "/etc/rc0.d/K%02d%s",
+        	   get_stop(dist->files + i, 0), dist->files[i].dst);
+        else
+	  snprintf(file->dst, sizeof(file->dst), "/etc/rc%c.d/S%02d%s",
+        	   *runlevels, get_start(dist->files + i, 99),
+		   dist->files[i].dst);
+      }
+
+     /*
+      * Then send the original file to /etc/init.d...
+      */
 
       file = dist->files + i;
 
@@ -389,27 +403,27 @@ make_pkg(const char     *prodname,	/* I - Product short name */
     {
       case 'c' :
           if (file->src[0] == '/')
-            fprintf(fp, "e none %s=%s %04o %s %s\n", file->dst,
+            qprintf(fp, "e none %s=%s %04o %s %s\n", file->dst,
 	            file->src, file->mode, file->user, file->group);
           else
-            fprintf(fp, "e none %s=%s/%s %04o %s %s\n", file->dst,
+            qprintf(fp, "e none %s=%s/%s %04o %s %s\n", file->dst,
 	            current, file->src, file->mode, file->user, file->group);
           break;
       case 'd' :
-	  fprintf(fp, "d none %s %04o %s %s\n", file->dst, file->mode,
+	  qprintf(fp, "d none %s %04o %s %s\n", file->dst, file->mode,
 		  file->user, file->group);
           break;
       case 'f' :
       case 'i' :
           if (file->src[0] == '/')
-            fprintf(fp, "f none %s=%s %04o %s %s\n", file->dst,
+            qprintf(fp, "f none %s=%s %04o %s %s\n", file->dst,
 	            file->src, file->mode, file->user, file->group);
           else
-            fprintf(fp, "f none %s=%s/%s %04o %s %s\n", file->dst,
+            qprintf(fp, "f none %s=%s/%s %04o %s %s\n", file->dst,
 	            current, file->src, file->mode, file->user, file->group);
           break;
       case 'l' :
-          fprintf(fp, "s none %s=%s\n", file->dst, file->src);
+          qprintf(fp, "s none %s=%s\n", file->dst, file->src);
           break;
     }
 
@@ -455,7 +469,8 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   if (Verbosity)
     puts("Copying into package stream file...");
 
-  if (run_command(directory, "pkgtrans -s . %s.pkg %s", name, prodname))
+  if (run_command(directory, "pkgtrans -s %s/%s %s.pkg %s",
+                  current, directory, name, prodname))
     return (1);
 
  /*
@@ -488,5 +503,5 @@ make_pkg(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: pkg.c,v 1.18 2001/06/26 16:22:22 mike Exp $".
+ * End of "$Id: pkg.c,v 1.23 2002/10/17 18:13:14 mike Exp $".
  */
