@@ -1,9 +1,9 @@
 //
-// "$Id: setup2.cxx,v 1.19 2001/07/05 16:22:06 mike Exp $"
+// "$Id: setup2.cxx,v 1.30 2002/06/04 19:17:31 mike Exp $"
 //
 //   ESP Software Wizard main entry for the ESP Package Manager (EPM).
 //
-//   Copyright 1999-2001 by Easy Software Products.
+//   Copyright 1999-2002 by Easy Software Products.
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include <FL/x.H>
 #include <FL/filename.H>
 #include <FL/fl_ask.H>
-#include <FL/Fl_Pixmap.H>
+#include <FL/Fl_XPM_Image.H>
 #include <errno.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -43,6 +43,10 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_PARAM_H
+#  include <sys/param.h>
+#endif // HAVE_SYS_PARAM_H
+
 #ifdef HAVE_SYS_MOUNT_H
 #  include <sys/mount.h>
 #endif // HAVE_SYS_MOUNT_H
@@ -91,6 +95,8 @@ main(int  argc,		// I - Number of command-line arguments
   Fl_Window	*w;	// Main window...
 
 
+  Fl::scheme(NULL);
+
   if (getuid() != 0)
   {
     fl_alert("You must be logged in as root to run setup!");
@@ -239,7 +245,7 @@ get_dists(const char *d)	// I - Directory to look in
     exit(1);
   }
 
-  if ((num_files = filename_list(".", &files)) == 0)
+  if ((num_files = fl_filename_list(".", &files)) == 0)
   {
     fputs("setup: Error - no software products found!\n", stderr);
     exit(1);
@@ -251,7 +257,7 @@ get_dists(const char *d)	// I - Directory to look in
 
   for (i = 0; i < num_files; i ++)
   {
-    ext = filename_ext(files[i]->d_name);
+    ext = fl_filename_ext(files[i]->d_name);
     if (strcmp(ext, ".install") == 0)
     {
       // Found a .install script...
@@ -365,13 +371,13 @@ get_installed(void)
   NumInstalled = 0;
   Installed    = NULL;
 
-  if ((num_files = filename_list(EPM_SOFTWARE, &files)) == 0)
+  if ((num_files = fl_filename_list(EPM_SOFTWARE, &files)) == 0)
     return;
 
   // Build a distribution list...
   for (i = 0; i < num_files; i ++)
   {
-    ext = filename_ext(files[i]->d_name);
+    ext = fl_filename_ext(files[i]->d_name);
     if (strcmp(ext, ".remove") == 0)
     {
       // Found a .remove script...
@@ -464,6 +470,7 @@ install_dist(const dist_t *dist)// I - Distribution to install
       Fl::wait();
 
     CancelButton->deactivate();
+    NextButton->deactivate();
 
     if (LicenseDecline->value())
     {
@@ -648,32 +655,12 @@ list_cb(Fl_Check_Browser *, void *)
 void
 load_image(void)
 {
-  FILE		*fp;		// Setup image file
-  char		line[1024];	// Line from file
-  Fl_Pixmap	*image;		// New image
-  int		num_lines;	// Number of lines in file
-  static char	*lines[1000];	// Pointer to lines
+  Fl_XPM_Image	*xpm;		// New image
 
 
-  if ((fp = fopen("setup.xpm", "r")) == NULL)
-    return;
+  xpm = new Fl_XPM_Image("setup.xpm");
 
-  num_lines = 0;
-  while (fgets(line, sizeof(line), fp) != NULL &&
-         num_lines < 1000)
-  {
-    if (line[0] != '\"')
-      continue;
-
-    line[strlen(line) - 3] = '\0';	/* Drop ",\n */
-    lines[num_lines] = strdup(line + 1);
-    num_lines ++;
-  }
-
-  fclose(fp);
-
-  image = new Fl_Pixmap(lines);
-  image->label(WelcomeImage);
+  WelcomeImage->image(xpm);
 }
 
 
@@ -864,7 +851,7 @@ next_cb(Fl_Button *, void *)
 
     CancelButton->activate();
 
-    XBell(fl_display, 100);
+    fl_beep();
 
     installing = 0;
   }
@@ -891,7 +878,7 @@ sort_dists(const dist_t *d0,	// I - First distribution
 //
 
 void
-type_cb(CheckButton *w, void *)	// I - Check button widget
+type_cb(Fl_Round_Button *w, void *)	// I - Radio button widget
 {
   int		i;		// Looping var
   dtype_t	*dt;		// Current install type
@@ -1016,21 +1003,21 @@ update_sizes(void)
     }
 
   // Get the sizes of the root and /usr partition...
-#if defined(__sgi) || defined(__sun)
+#if defined(__sgi) || defined(__svr4__) || defined(__SVR4) || defined(M_XENIX)
   if (statfs("/", &rootpart, sizeof(rootpart), 0))
 #else
   if (statfs("/", &rootpart))
-#endif // __sgi || __sun
+#endif // __sgi || __svr4__ || __SVR4 || M_XENIX
     rootfree = 1024;
   else
     rootfree = (int)((double)rootpart.f_bfree * (double)rootpart.f_bsize /
                      1024.0 / 1024.0 + 0.5);
 
-#if defined(__sgi) || defined(__sun)
+#if defined(__sgi) || defined(__svr4__) || defined(__SVR4) || defined(M_XENIX)
   if (statfs("/usr", &usrpart, sizeof(usrpart), 0))
 #else
   if (statfs("/usr", &usrpart))
-#endif // __sgi || __sun
+#endif // __sgi || __svr4__ || __SVR4 || M_XENIX
     usrfree = 1024;
   else
     usrfree = (int)((double)usrpart.f_bfree * (double)usrpart.f_bsize /
@@ -1077,5 +1064,5 @@ update_sizes(void)
 
 
 //
-// End of "$Id: setup2.cxx,v 1.19 2001/07/05 16:22:06 mike Exp $".
+// End of "$Id: setup2.cxx,v 1.30 2002/06/04 19:17:31 mike Exp $".
 //
