@@ -1,9 +1,9 @@
 /*
- * "$Id: swinstall.c,v 1.14 2001/06/26 16:22:22 mike Exp $"
+ * "$Id: swinstall.c,v 1.21 2002/10/17 18:13:14 mike Exp $"
  *
  *   HP-UX package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2001 by Easy Software Products.
+ *   Copyright 1999-2002 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
   file_t	*file;			/* Current distribution file */
   command_t	*c;			/* Current command */
   depend_t	*d;			/* Current dependency */
+  const char	*runlevels;		/* Run levels */
 
 
   (void)platform; /* Eliminates compiler warning about unused variable */
@@ -73,39 +74,6 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
   else
     snprintf(name, sizeof(name), "%s-%s", prodname, dist->version);
 
- /*
-  * Add symlinks for init scripts...
-  */
-
-  for (i = 0; i < dist->num_files; i ++)
-    if (tolower(dist->files[i].type) == 'i')
-    {
-      file = add_file(dist);
-      file->type = 'l';
-      file->mode = 0;
-      strcpy(file->user, "root");
-      strcpy(file->group, "sys");
-      snprintf(file->src, sizeof(file->src), "../init.d/%s",
-               dist->files[i].dst);
-      snprintf(file->dst, sizeof(file->dst), "/sbin/rc0.d/K000%s",
-               dist->files[i].dst);
-
-      file = add_file(dist);
-      file->type = 'l';
-      file->mode = 0;
-      strcpy(file->user, "root");
-      strcpy(file->group, "sys");
-      snprintf(file->src, sizeof(file->src), "../init.d/%s",
-               dist->files[i].dst);
-      snprintf(file->dst, sizeof(file->dst), "/sbin/rc2d.d/S999%s",
-               dist->files[i].dst);
-
-      file = dist->files + i;
-
-      snprintf(filename, sizeof(filename), "/sbin/init.d/%s", file->dst);
-      strcpy(file->dst, filename);
-    }
-
  /*
   * Write the preinstall script if needed...
   */
@@ -142,10 +110,6 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
       if (c->type == COMMAND_PRE_INSTALL)
         fprintf(fp, "%s\n", c->command);
 
-    for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
-	fprintf(fp, "/sbin/init.d/%s start\n", file->dst);
-
     fclose(fp);
   }
   else
@@ -194,7 +158,7 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
       if (tolower(file->type) == 'i')
-	fprintf(fp, "/sbin/init.d/%s start\n", file->dst);
+	qprintf(fp, "/sbin/init.d/%s start\n", file->dst);
 
     fclose(fp);
   }
@@ -239,7 +203,7 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 
     for (j = dist->num_files, file = dist->files; j > 0; j --, file ++)
       if (tolower(file->type) == 'i')
-	fprintf(fp, "/sbin/init.d/%s stop\n", file->dst);
+	qprintf(fp, "/sbin/init.d/%s stop\n", file->dst);
 
     for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
       if (c->type == COMMAND_PRE_REMOVE)
@@ -286,15 +250,53 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
       if (c->type == COMMAND_POST_REMOVE)
         fprintf(fp, "%s\n", c->command);
 
-    for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
-	fprintf(fp, "/sbin/init.d/%s start\n", file->dst);
-
     fclose(fp);
   }
   else
     postremove[0] = '\0';
 
+ /*
+  * Add symlinks for init scripts...
+  */
+
+  for (i = 0; i < dist->num_files; i ++)
+    if (tolower(dist->files[i].type) == 'i')
+    {
+     /*
+      * Make symlinks for all of the selected run levels...
+      */
+
+      for (runlevels = get_runlevels(dist->files + i, "02");
+           isdigit(*runlevels);
+	   runlevels ++)
+      {
+	file = add_file(dist);
+	file->type = 'l';
+	file->mode = 0;
+	strcpy(file->user, "root");
+	strcpy(file->group, "sys");
+	snprintf(file->src, sizeof(file->src), "../init.d/%s",
+        	 dist->files[i].dst);
+
+        if (*runlevels == '0')
+	  snprintf(file->dst, sizeof(file->dst), "/sbin/rc0.d/K%02d0%s",
+        	   get_stop(dist->files + i, 0), dist->files[i].dst);
+        else
+	  snprintf(file->dst, sizeof(file->dst), "/sbin/rc%c.d/S%02d0%s",
+        	   *runlevels, get_start(dist->files + i, 99),
+		   dist->files[i].dst);
+      }
+
+     /*
+      * Then send the original file to /sbin/init.d...
+      */
+
+      file = dist->files + i;
+
+      snprintf(filename, sizeof(filename), "/sbin/init.d/%s", file->dst);
+      strcpy(file->dst, filename);
+    }
+
  /*
   * Create all symlinks...
   */
@@ -349,9 +351,9 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 
   if (i)
   {
-    fputs("    corequisites", fp);
-
     for (; i > 0; i --, d ++)
+    {
+      fputs("    prerequisites", fp);
       if (d->type == DEPEND_REQUIRES && d->product[0] != '/')
       {
         fprintf(fp, " %s", d->product);
@@ -363,8 +365,8 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 	else
 	  fprintf(fp, ",r>=%s,r<=%s", d->version[0], d->version[1]);
       }
-
-    fputs("\n", fp);
+      fputs("\n", fp);
+    }
   }
 
   for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
@@ -373,9 +375,9 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 
   if (i)
   {
-    fputs("    ancestor", fp);
-
     for (; i > 0; i --, d ++)
+    {
+      fputs("    ancestor", fp);
       if (d->type == DEPEND_REPLACES && d->product[0] != '/')
       {
         fprintf(fp, " %s", d->product);
@@ -387,8 +389,8 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 	else
 	  fprintf(fp, ",r>=%s,r<=%s", d->version[0], d->version[1]);
       }
-
-    fputs("\n", fp);
+      fputs("\n", fp);
+    }
   }
 
   if (preinstall[0])
@@ -407,23 +409,23 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
     switch (tolower(file->type))
     {
       case 'd' :
-          fprintf(fp, "    file -m %o -o %s -g %s . %s\n", file->mode | S_IFDIR,
+          qprintf(fp, "    file -m %o -o %s -g %s . %s\n", file->mode | S_IFDIR,
 	          file->user, file->group, file->dst);
           break;
       case 'c' :
-          fprintf(fp, "    file -m %04o -o %s -g %s -v %s %s\n", file->mode,
+          qprintf(fp, "    file -m %04o -o %s -g %s -v %s %s\n", file->mode,
 	          file->user, file->group, file->src, file->dst);
           break;
       case 'f' :
       case 'i' :
-          fprintf(fp, "    file -m %04o -o %s -g %s %s %s\n", file->mode,
+          qprintf(fp, "    file -m %04o -o %s -g %s %s %s\n", file->mode,
 	          file->user, file->group, file->src, file->dst);
           break;
       case 'l' :
           snprintf(filename, sizeof(filename), "%s/%s.link%04d", directory,
 	           prodname, linknum);
           linknum ++;
-          fprintf(fp, "    file -o %s -g %s %s %s\n", file->user, file->group,
+          qprintf(fp, "    file -o %s -g %s %s %s\n", file->user, file->group,
 	          filename, file->dst);
           break;
     }
@@ -506,5 +508,5 @@ make_swinstall(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: swinstall.c,v 1.14 2001/06/26 16:22:22 mike Exp $".
+ * End of "$Id: swinstall.c,v 1.21 2002/10/17 18:13:14 mike Exp $".
  */
